# -*- coding: utf-8 -*-
import os, os.path, sys, fnmatch

install_prefix = "/tmp"

IGNORE_MAP = True

##
## Recursive funcs
##
def GetFiles(env, dir, includes, excludes=None):
      files = []
      for file in os.listdir(env.Dir(dir).srcnode().abspath):
          for pattern in includes:
             if fnmatch.fnmatchcase(file, pattern):
                 files.append(file)
      if not excludes is None:
        for file in files:
           for pattern in excludes:
              if fnmatch.fnmatchcase(file, pattern):
                  files.remove(file)
      return files

def InstallFiles(env, dest_dir, src_dir, includes, excludes, mapfiles):
    if "settings.py" not in excludes:
        #settings.py sources are ALWAYS excluded
        excludes.append("settings.py")
    src = GetFiles(env, src_dir, includes, excludes)
    #print "x=" + `src`
    dest = env.Dir(dest_dir)
    result = []
    for f in src:
        if f in mapfiles:
            # apply mapping on installation
            result.append(env.InstallAsPerm(env, env.File(os.path.join(dest_dir,mapfiles[f])), env.File(os.path.join(src_dir,f)),0664))
        
        if IGNORE_MAP:
            result.append(env.InstallPerm(env, dest, os.path.join(src_dir,f), 0664))
        else:
            if f not in mapfiles.values():                          # if it is in, we should NOT install, because another file will be installed into this position
                result.append(env.InstallPerm(env, dest, os.path.join(src_dir,f), 0664))
    
    return result

def RecursiveInstallFiles(env, dest_dir, src_dir, includes, excludes, mapfiles={}):
    """Recursively install all the source directories into dest in the same direectory hierachy
    mapfiles should be a hash with src:dest filepath matching to move files into their destination but under a different name
    eg: {'settings-live.py':'settings.py', 'static/js/ie.css':'static/js/css.css'}
    Paths are handled correctly thoughout the recursion.
    """
    #print "RIF",env,dest_dir,src_dir,includes,excludes,"..."
    parts = os.listdir( str(src_dir) )           # all the files and directories in this directory
    installer = []
    
    # only pass through the mapfiles whos file keys relate to the present directory
    installmap = {}
    for s,d in mapfiles.iteritems():
        if len(os.path.normpath(s).split(os.path.sep)) == 1:
            installmap[s]=d
    installer.extend(InstallFiles(env, dest_dir, src_dir, includes, excludes, mapfiles=installmap))
    
    # recurse into directories
    for part in parts:
        if os.path.isdir(os.path.join(str(src_dir),part)) and part not in excludes and part!="tmp":
            # path mangle mapfiles to get ready for the recurse into this directory.
            subdir_mapfiles = {}
            for s,d in mapfiles.iteritems():
                pathparts = os.path.normpath(s).split(os.path.sep)
                if len(pathparts) > 1:
                    if pathparts[0]==part:
                        # this applies to our subdirectory
                        subdir_mapfiles[os.path.sep.join(pathparts[1:])]=d
            
            results = RecursiveInstallFiles(env, os.path.join(str(dest_dir),part), os.path.join(str(src_dir),part), includes, excludes, mapfiles=subdir_mapfiles)
            installer += results
    
    #print [str(s) for s in installer]
    return installer

def InstallTree(env, dest_dir, src_dir, includes, excludes):
    destnode = env.Dir(dest_dir)
    dirs = []
    dirs.append(src_dir)
    while len(dirs) > 0:
      currdir = dirs.pop(0)
      currdestdir = dest_dir + currdir[len(src_dir):]
      #print "c=" + currdestdir
      flist = os.listdir(currdir)
      for currfile in flist:
         currpath = currdir + '/' + currfile
         match = 0
         for pattern in includes:
            if fnmatch.fnmatchcase(currfile, pattern):
              match = 1
         if (match == 1):
            for pattern in excludes:
               if fnmatch.fnmatchcase(currfile, pattern):
                  match = 0
            if (match == 1):
               if (os.path.isdir(currpath)):
                  #print "d=" + currpath
                  dirs.append(currpath)
               else:
                  #print "f=" + currpath
                  env.Install(currdestdir, currpath)
      #print "x= len=" + str(len(dirs))
    return destnode

##
## CCG stuff
##
def virtual_python(target, source, env):
    """Installs virtual python in target, then easy install, then each requested egg"""
    for t in target:
        # install virtual python
        command = 'python  /usr/local/etc/ccgbuild/virtual-python.py --no-site-packages --prefix=%s' % (t)
        print command
        print os.popen(command).read()
        
        # install ez_setup
        pythonbin = os.path.join(str(t),"bin","python")
        command = '%s /usr/local/etc/ccgbuild/ez_setup.py' % (pythonbin)
        print command
        print os.popen(command).read()
        
        # install each egg
        pythonbin = os.path.join(str(t),"bin","easy_install")
        for egg in source:
            command = '%s %s' % (pythonbin,egg)
            print command
            print os.popen(command).read()
        
    return None

def apache_wsgi_file(target, source, env):
    """Writes into the target WSGI file, the relevant commands"""
    source = source[0]
    for t in target:
        fname = os.path.basename(str(t))
        base,tag = fname.rsplit(":",1)          # base is the project name. tag is the release tag, or 'trunk' or 'release'
        
        fh = open(str(t),'w')
        fh.write("WSGIScriptAlias /%s/%s %s/%s/%s/%s/%s\n"%(base,tag,install_prefix,base,tag,base,source))
        fh.close()
        
    return None
    
def apache_alias_file(target, source, env):
    """Writes into the target WSGI file, the relevant commands. sources are our server directories"""
    SUFFIX = "-aliases"
    for t in target:
        assert str(t).endswith(SUFFIX)
        fname = os.path.basename(str(t))[:-len(SUFFIX)]
        base,tag = fname.rsplit(":",1)          # base is the project name. tag is the release tag, or 'trunk' or 'release'
        
        fh = open(str(t),'w')
        for s in source:
            fh.write("Alias /%s/%s/%s %s/%s/%s/%s/%s\n"%(base,tag,s,install_prefix,base,tag,base,s))
        fh.close()
        
    return None
    
    
def symlink(target, source, env):
    """Note: target is the symlink itself
    source is what it points at"""
    src = str(source[0])
    for t in target:
        prefix = os.path.commonprefix([os.path.dirname(str(src)),os.path.dirname(str(t))]) + "/"
        source = str(src)[len(prefix):]
        os.symlink(source,str(t))
        
    return None
    
def InstallPerm(env, dest, files, perm):
    obj = env.Install(dest, files)
    for i in obj:
        env.AddPostAction(i, env.Chmod(str(i), perm))
    return dest

def InstallAsPerm(env, dest, files, perm):
    obj = env.InstallAs(dest, files)
    for i in obj:
        env.AddPostAction(i, env.Chmod(str(i), perm))
    return dest

##
## Build our environment
##
def EnvironmentFactory( env, Builder, environ ):
    vpb = Builder(action=virtual_python)
    awb = Builder(action=apache_wsgi_file)
    apa = Builder(action=apache_alias_file)
    slb = Builder(action=symlink)
    e = env( ENV = environ, BUILDERS = { 'VirtualPython':vpb, "ApacheWSGI":awb, "ApacheAlias":apa, "SymLink":slb } )
    import SCons
    e.Chmod = SCons.Action.ActionFactory(os.chmod, lambda dest, mode: 'Chmod("%s", 0%o)' % (dest, mode))
    e.InstallPerm = InstallPerm
    e.InstallAsPerm = InstallAsPerm
    return e
